# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2008-2010 Anevia S.A. (www.anevia.com),
#    @author Raphaël Valyi <rvalyi@akretion.com>
#    @author Alexis de Lattre <alexis.delattre@akretion.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv

class crm_claim(osv.osv):
    _inherit = "crm.claim"

    def _get_claim_from_stock_location(self, cr, uid, ids, context=None):
        return self.pool.get('crm.claim').search(cr, uid, [('fleet_id', 'in', ids)], context=context)

    _columns = {
        # We have a sequence on crm claims : should we continue to use our own sequence for RMAs ?
        'incident_ref': fields.char('RMA ref', size=64, required=True),
        'external_ref': fields.char('Ticket number', size=64),
        'fleet_id': fields.many2one('stock.location', 'Fleet'),
        'parent_fleet_id': fields.related('fleet_id', 'location_id', type='many2one', relation='stock.location', string='Fleet', store={
            'crm.claim': (lambda self, cr, uid, ids, c={}: ids, ['fleet_id'], 10),
            'stock.location': (_get_claim_from_stock_location, ['location_id'], 20),
            }),
        'is_fleet_under_maintenance': fields.related('fleet_id', 'is_under_maintenance', type='boolean', string='Fleet is under maintenance'),
        # Think about it : we would probably like to keep the is_under_maintenance at the date on which we open the CRM case
        'repair_picking_id': fields.many2one('stock.picking', 'Repair picking'),
        'incoming_picking_id': fields.many2one('stock.picking', 'Incoming picking'),
        'outgoing_picking_id': fields.many2one('stock.picking', 'Outgoing picking'),
        'related_repair_picking_state': fields.related('repair_picking_id', 'state', type="char", string="State of repair picking", readonly=True),
        'related_incoming_picking_state': fields.related('incoming_picking_id', 'state', type="char", string="State of incoming picking", readonly=True),
        'related_outgoing_picking_state': fields.related('outgoing_picking_id', 'state', type="char", string="State of outgoing picking", readonly=True),
        'related_repair_picking_date': fields.related('repair_picking_id', 'date_done', type="datetime", string="Date of repair picking", readonly=True),
        'related_incoming_picking_date': fields.related('incoming_picking_id', 'date_done', type="datetime", string="Date of incoming picking", readonly=True),
        'related_outgoing_picking_date': fields.related('outgoing_picking_id', 'date_done', type="datetime", string="Date of outgoing picking", readonly=True),
        # The name of the 2 fields below is tricky :
        # in_supplier_picking_id is the mvt Company -> Supplier, so it's an OUT
        # out_supplier_picking_id is the mvt Supplier -> Company, so it's an IN
        'in_supplier_picking_id': fields.many2one('stock.picking', 'Return to supplier picking'),
        'out_supplier_picking_id': fields.many2one('stock.picking', 'Return from supplier picking'),
        'related_in_supplier_picking_state': fields.related('in_supplier_picking_id', 'state', type="char", string="State of return to supplier picking", readonly=True),
        'related_out_supplier_picking_state': fields.related('out_supplier_picking_id', 'state', type="char", string="State of return from supplier picking", readonly=True),
        'related_in_supplier_picking_date': fields.related('in_supplier_picking_id', 'date_done', type="datetime", string="Date of return to supplier picking", readonly=True),
        'related_out_supplier_picking_date': fields.related('out_supplier_picking_id', 'date_done', type="datetime", string="Date of return from supplier picking", readonly=True),
        'prodlot_id': fields.many2one('stock.production.lot', 'Serial number'),
        'product_id': fields.related('prodlot_id', 'product_id', type='many2one', relation='product.product', string='Related product', store={
            'crm.claim': (lambda self, cr, uid, ids, c={}: ids, ['prodlot_id'], 10),
        }),
    }

    _defaults = {
        'incident_ref': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'crm.rma'),
        }


    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default.update({
            'incident_ref': self.pool.get('ir.sequence').get(cr, uid, 'crm.rma'),
            })
        return super(crm_claim, self).copy(cr, uid, id, default, context=context)


    def onchange_prodlot_id(self, cr, uid, ids, prodlot_id):
        result = {}
        result['value'] = {}
        if not prodlot_id:
            return result

        # TODO This is a copy of the fields.function "last_location" of product_serial
        cr.execute(
                "select location_dest_id " \
                "from stock_move inner join stock_report_prodlots on stock_report_prodlots.location_id = location_dest_id and stock_report_prodlots.prodlot_id = %s " \
                "where stock_move.prodlot_id = %s and stock_move.state=%s "\
                "order by stock_report_prodlots.qty DESC ",
                (prodlot_id, prodlot_id, 'done'))

        results = cr.fetchone()

        if results and len(results) > 0:
            sub_fleet = self.pool.get('stock.location').browse(cr, uid, results[0])
            product_id = self.pool.get('stock.production.lot').browse(cr, uid, prodlot_id).product_id.id
            result['value'].update({
                'fleet_id': sub_fleet.id,
                'partner_id': sub_fleet.parent_partner_id.id,
                'is_fleet_under_maintenance': sub_fleet.is_under_maintenance,
                'product_id': product_id
                })
        else:
            result['value'].update({'fleet_id': False, 'partner_id': False,
                'is_fleet_under_maintenance': False})

        return result

