# -*- encoding: utf-8 -*-
##############################################################################
#
#    Fleet maintenance module for OpenERP
#    Copyright (C) 2008-2010 Anevia S.A. (www.anevia.com),
#       code written by Raphaêl Valyi <rvalyi@akretion.com>
#    Copyright (C) 2010 Alexis de Lattre <alexis@via.ecp.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields,osv
from mx.DateTime import RelativeDateTime
from mx import DateTime
from onchange import fleet_maintenance_osv


fixed_month_init_day = 1 # TODO make this a parameter!
fixed_days_before_month_end = 0 #TODO make this a parameter!
min_maintenance_months = 6 #TODO make this a parameter!


class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def _refund_cleanup_lines(self, cr, uid, lines):
        for line in lines:
            if 'fleet_id' in line:
                line['fleet_id'] = line.get('fleet_id', False) and line['fleet_id'][0]
            if 'parent_fleet_id' in line:
                line['parent_fleet_id'] = line.get('parent_fleet_id', False) and line['parent_fleet_id'][0]
            if 'account_analytic_lines' in line:
                line['account_analytic_lines'] = [(6,0, line.get('account_analytic_lines', [])) ]
        return super(account_invoice, self)._refund_cleanup_lines(cr, uid, lines)

    def inv_line_characteristic_hashcode(self, invoice, invoice_line, context=None):
        code = super(account_invoice, self).inv_line_characteristic_hashcode(invoice, invoice_line)
        code += '-'+str(invoice_line.get('maintenance_start_date',"False"))
        code += '-'+str(invoice_line.get('maintenance_end_date',"False"))
        code += '-'+str(invoice_line.get('fleet_id',"False"))
        return code

    def line_get_convert(self, cr, uid, x, part, date, context=None):
        result = super(account_invoice, self).line_get_convert(cr, uid, x, part, date, context=context)
        result['maintenance_start_date'] = x.get('maintenance_start_date', False)
        result['maintenance_end_date'] = x.get('maintenance_end_date', False)
        result['fleet_id'] = x.get('fleet_id', False)
        return result

    def update_with_defaultfleet(self, cr, uid, ids, context=None):
        return self.pool.get('account.invoice.line')._common_update_with_defaultfleet(cr, uid, self, ids, context=context)

    _columns = {
        'default_fleet_id': fields.many2one('stock.location', 'Default sub-fleet', help="Select the sub-fleet which will be used by default in the new invoice lines."),
    }

account_invoice()


class account_invoice_line(fleet_maintenance_osv):
    _inherit = "account.invoice.line"

    def move_line_get_item(self, cr, uid, line, context=None):
        result = super(account_invoice_line, self).move_line_get_item(cr, uid, line, context=context)
        result['maintenance_start_date'] = line.maintenance_start_date
        result['maintenance_end_date'] = line.maintenance_end_date
        result['fleet_id'] = line.fleet_id
        return result

    def create(self, cr, uid, vals, context=None):
        """Prevents the ORM from trying to write the is_maintenance product fields view fields related.
        User might indeed not have this right, example is an accountant creating a credit note.
        Actually this looks like a bug in OpenObject because it will try to write even if we declared
        the is_maintenance field as read_only.
        see bugreport here https://bugs.launchpad.net/openobject-server/+bug/426676
        """
        if vals.get('is_maintenance', False):
            del(vals['is_maintenance'])
        if vals.get('account_analytic_lines', False):
            del(vals['account_analytic_lines'])
        return super(account_invoice_line, self).create(cr, uid, vals, context=context)

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default['account_analytic_lines'] = False
        return super(account_invoice_line, self).copy(cr, uid, id, default, context=context)

    def _function_maintenance_month_qty(self, cr, uid, ids, prop, unknow_none, context=None):
        '''Used for definition of maintenance_month_qty field, which is a fields.function'''
        return self.function_maintenance_month_qty(cr, uid, ids, context=context)

    def _function_subfleet_required(self, cr, uid, ids, prop, unknow_none, context=None):
        '''Used for definition of subfleet_required field, which is a fields.function'''
        return self.function_subfleet_required(cr, uid, ids, context=context)


    def maintenance_qty_date_change(self, cr, uid, ids, maintenance_product_qty=False, maintenance_month_qty=False, maintenance_start_date=False, maintenance_end_date=False, is_maintenance=False, fleet_id=False, context=None):
        params = {'maintenance_product_qty': maintenance_product_qty, 'maintenance_month_qty': maintenance_month_qty, 'maintenance_start_date': maintenance_start_date, 'maintenance_end_date': maintenance_end_date, 'is_maintenance': is_maintenance, 'fleet_id': fleet_id}
        return self.common_maintenance_qty_date_change(cr, uid, ids, params, context=context)


    def product_id_change(self, cr, uid, ids, product_id, uom, qty=0, name='', type='out_invoice', partner_id=False, fposition_id=False, price_unit=False, address_invoice_id=False, currency_id=False, company_id=None, is_maintenance=False, maintenance_product_qty=False, maintenance_month_qty=False, default_fleet_id=False, subfleet_required=False, context=None):
        result = super(account_invoice_line, self).product_id_change(cr, uid, ids, product_id, uom, qty=qty, name=name, type=type, partner_id=partner_id, fposition_id=fposition_id, price_unit=price_unit, address_invoice_id=address_invoice_id, currency_id=currency_id, company_id=company_id, context=context)

        params = {'product_id': product_id, 'is_maintenance': is_maintenance, 'maintenance_product_qty': maintenance_product_qty, 'maintenance_month_qty': maintenance_month_qty, 'default_fleet_id': default_fleet_id, 'subfleet_required': subfleet_required, 'invoice_type': type}
        result = self.common_product_id_change(cr, uid, ids, result, params, context=context)
        return result


    def fleet_id_change(self, cr, uid, ids, is_maintenance=False, default_fleet_id=False, fleet_id=False, product_id=False, maintenance_start_date=False, maintenance_product_qty=False, subfleet_required=False, context=None):
        params = {'is_maintenance': is_maintenance, 'default_fleet_id': default_fleet_id, 'fleet_id': fleet_id, 'product_id': product_id, 'maintenance_start_date': maintenance_start_date, 'maintenance_product_qty': maintenance_product_qty, 'subfleet_required': subfleet_required}
        return self.common_fleet_id_change(cr, uid, ids, params, context=context)

    def _get_invoice_line_from_stock_location(self, cr, uid, ids, context=None):
        return self.pool.get('account.invoice.line').search(cr, uid, [('fleet_id', 'in', ids)], context=context)

    _columns = {
        'maintenance_product_qty': fields.integer('Maintenance Product Quantity', help="Number of 'real' products covered by this maintenance product."),
        'maintenance_month_qty': fields.function(_function_maintenance_month_qty, string="Maintenance Month Quantity", type='integer', store={
            'account.invoice.line': (lambda self, cr, uid, ids, c={}: ids, ['maintenance_start_date', 'maintenance_end_date'], 10),
            }, help="Number of months of maintenance."),
        'fleet_id': fields.many2one('stock.location', 'Sub Fleet', help="For a 'real' product : this is the sub-fleet where the product are located. For a 'maintenance product' : this is the sub-fleet which is covered by the maintenance."),
        'parent_fleet_id': fields.related('fleet_id', 'location_id', type='many2one', relation='stock.location', string='Fleet', store={
            'account.invoice.line': (lambda self, cr, uid, ids, c={}: ids, ['fleet_id'], 10),
            'stock.location': (_get_invoice_line_from_stock_location, ['location_id'], 20),
            }), # Not displayed on screen
        # We need store=True on parent_fleet_id, because othewise the field
        # account_invoice_lines on stock.location doesn't work
        'maintenance_start_date':fields.date('Maintenance Start Date', help="Date when the maintenance starts."),
        'maintenance_end_date':fields.date('Maintenance End Date', help="Date when the maintenance stops."),
        'default_fleet_id': fields.related('invoice_id', 'default_fleet_id', type='many2one', relation='stock.location', string='Default invoice sub-fleet'), # Not displayed on screen
        'is_maintenance': fields.related('product_id', 'is_maintenance', type='boolean', string='Is Maintenance'), # Not displayed on screen
        'subfleet_required': fields.function(_function_subfleet_required, string="Subfleet required", type='boolean', store={
            'account.invoice.line': (lambda self, cr, uid, ids, c={}: ids, ['product_id'], 10),
            }), # Not displayed on screen
        'account_analytic_lines': fields.one2many('account.analytic.line', 'invoice_line_id', 'Analytic Lines', readonly=True),
    }

    def _check_maintenance_all(self, cr, uid, ids, context=None):
        return self.contraints_check_maintenance_all(cr, uid, ids, context=context)

    _constraints = [
        (_check_maintenance_all, """We don't use this message. We raise an error with the message inside""", ['quantity', 'maintenance_product_qty', 'maintenance_month_qty', 'maintenance_start_date', 'maintenance_end_date', 'fleet_id', 'product_id']),
    ]

account_invoice_line()


class account_analytic_line(osv.osv):
    _inherit = 'account.analytic.line'

    _columns = {
        'invoice_line_id': fields.many2one('account.invoice.line', 'Invoice Line'),
        'invoice_id': fields.related('invoice_line_id', 'invoice_id', type='many2one', relation='account.invoice', string='Invoice', store=True, select=1),
    }

account_analytic_line()


class account_move_line(osv.osv):
    _inherit="account.move.line"

    def create_analytic_lines(self, cr, uid, ids, context=None):
        if context == None:
            context = {}
        res = super(account_move_line, self).create_analytic_lines(cr, uid, ids, context=context)
        matched_invoice_line_ids = []
        cur_obj = self.pool.get('res.currency')
        current_user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_currency_id = current_user.company_id.currency_id.id
        for move_line in self.browse(cr, uid, ids, context=context):
            if move_line.analytic_lines and len(move_line.analytic_lines) == 1 and move_line.product_id.is_maintenance and move_line.invoice:
                analytic_line = move_line.analytic_lines[0] #we assume their is only one analytic line for a maintenance product
                context['date'] = move_line.invoice.date_invoice
                for invoice_line in move_line.invoice.invoice_line:
                    if invoice_line.id not in matched_invoice_line_ids: #we take care of not matching invoice line twice
                    #moreover we assume that there is only on analytic line per move_line and so only one analytic line can be modificated, it why the loop it stop after changing the analytic line
                        if invoice_line.product_id and invoice_line.product_id.is_maintenance:
                            invoice_line_amount = cur_obj.compute(cr, uid, invoice_line.invoice_id.currency_id.id, company_currency_id, invoice_line.price_subtotal, context=context)
                            if 'refund' in invoice_line.invoice_id.type:
                                sign = -1
                            else:
                                sign = 1
                            if invoice_line.product_id.id == analytic_line.product_id.id and abs(analytic_line.amount - invoice_line_amount * sign)<0.001:
                                matched_invoice_line_ids.append(invoice_line.id)
                                month_qty = int(invoice_line.maintenance_month_qty)
                                splitted_amount =  invoice_line.quantity / invoice_line.maintenance_month_qty
                                amount = float(analytic_line.amount) / float(invoice_line.maintenance_month_qty)

                                start_date = DateTime.strptime(invoice_line.maintenance_start_date, '%Y-%m-%d')
                                self.pool.get('account.analytic.line').write(cr, uid, analytic_line.id, {'invoice_line_id':invoice_line.id, 'date': invoice_line.maintenance_start_date, 'unit_amount':splitted_amount, 'amount': amount})

                                for i in range(1, month_qty):
                                    start_date += DateTime.RelativeDateTime(months=1)
                                    start_date_str = start_date.strftime('%Y-%m-%d')
                                    self.pool.get('account.analytic.line').copy(cr, uid, analytic_line.id, {'date': start_date_str}) #TODO check month_qty
                                break

        return res

    #This code will regenerate all analytic lines. This is use for data migration
    def update_all_line(self, cr, uid):
        cur_obj = self.pool.get('res.currency')

        # ANALYTICS LINE NEED TO GET THE LINK BETWEEN THE ACCOUNT_INVOICE_LINE_ID AND THE ACCOUNT_MOVE_LINE
        # there is no way to get this link easily
        # so we will save the existing information from the analytics line in a dictionnary before deleting them
        # after with various method the relation will be found
           
        # start the script
        #import xmlrpclib
        #sock = xmlrpclib.ServerProxy('http://openerptrunk:7472/xmlrpc/object')
        #sock.execute("erp_ticket472", 1, "admin", 'account.move.line', 'update_all_line')

        cr.execute("select id from account_invoice where type in ('out_invoice', 'out_refund') and state in ('open', 'paid')")
        inv_ids = cr.fetchall()
        inv_ids = [x[0] for x in inv_ids]
        #print 'inv_id', inv_ids, len(inv_ids)
        analytic_line_obj = self.pool.get('account.analytic.line')
        analytic_line_ids = analytic_line_obj.search(cr, uid, [("invoice_id", "in", inv_ids)])
        analytic_line = analytic_line_obj.read(cr, uid, analytic_line_ids, ['invoice_line_id', 'move_id'])

        #READ and SAVE for each analytics line the link between the account_move_line and the account_invoice_line
        map={}
        for vals in analytic_line:
            map.update({vals['invoice_line_id'][0]:vals['move_id'][0]})
        #DELEDE ALL analytics lines select the YOUR journal id
        ids_to_del = analytic_line_obj.search(cr, uid, [('journal_id', '=', 1)])
        analytic_line_obj.unlink(cr, uid, ids_to_del)

        inv_line_obj = self.pool.get('account.invoice.line')
        inv_lines_ids = inv_line_obj.search(cr, uid, [('invoice_id', 'in', inv_ids)])


        matched_invoice_line_ids = []
        count = 0

        for invoice_line in inv_line_obj.browse(cr, uid, inv_lines_ids):
            if invoice_line.id not in matched_invoice_line_ids: #we take care of not matching invoice line twice
                if invoice_line.product_id and invoice_line.product_id.is_maintenance:
                    matched_invoice_line_ids.append(invoice_line.id)
                    month_qty = int(invoice_line.maintenance_month_qty)
                    splitted_amount =  invoice_line.quantity / invoice_line.maintenance_month_qty
                    context = {'date': invoice_line.invoice_id.date_invoice}
                    invoice_amount = cur_obj.compute(cr, uid, invoice_line.invoice_id.currency_id.id, invoice_line.account_analytic_id.company_currency_id.id, invoice_line.price_subtotal, context=context)
                    amount = float(invoice_amount) / float(invoice_line.maintenance_month_qty)
                    if invoice_line.invoice_id.type == 'out_refund':
                        amount = -amount

                    start_date = DateTime.strptime(invoice_line.maintenance_start_date, '%Y-%m-%d')


            #THIS code will find the account_move_id linked to the account_move_line no easy way to do it ;)
                    move_line_id = self.pool.get('account.move.line').search(cr, uid, [('invoice', '=', invoice_line.invoice_id.id), ('product_id', '=', invoice_line.product_id.id), ('quantity', '=', invoice_line.quantity)])

                    # GET the move_line_id from the dictionnary map or from the search done before
                    if map.get(invoice_line.id, False) or len(move_line_id)==1:
                        move_line_id = map.get(invoice_line.id, False) or move_line_id[0]
                    #If many move_line are found we used the move_line which don't have any analytics lines
                    elif len(move_line_id) >0:
                        analytic_line_ids = analytic_line_obj.search(cr, uid, [('move_id', 'in', move_line_id)])
                        move_line_ids_done = [x['move_id'] for x in analytic_line_obj.read(cr, uid, analytic_line_ids, ['move_id'])]
                        move_line_id = [x for x in move_line_id if x not in move_line_ids_done][0]
                        if not move_line_id:
                            raise
                    # If no move_line was found, it's because you used the option group invoice lines so a search is done based on the account_id
                    else:
                        #print 'invoice id', invoice_line.invoice_id.id, invoice_line.invoice_id.name
                        #print 'product', invoice_line.product_id.id, invoice_line.product_id.name
                        #print 'invoice_line.product_id.property_account_income.id', invoice_line.product_id.property_account_income.id
                        move_line_id = self.pool.get('account.move.line').search(cr, uid, [('invoice', '=', invoice_line.invoice_id.id), ('account_id', '=', invoice_line.account_id.id)])[0]

            #The move_line_id is found


                    if not invoice_line.account_analytic_id:
                        #print 'bug invoice line, the analytic account is not present! FIX IT', invoice_line.id, invoice_line.invoice_id.name, invoice_line.name
                        raise

                    vals_lines={
                        'name': invoice_line.invoice_id.partner_id.name + ' - ' + (invoice_line.invoice_id.name or ''),
                        'account_id': invoice_line.account_analytic_id.id,
                        'unit_amount': splitted_amount,
                        'product_id': invoice_line.product_id and invoice_line.product_id.id or False,
                        'product_uom_id': invoice_line.uos_id and invoice_line.uos_id.id or False,
                        'amount': amount,
                        'general_account_id': invoice_line.account_id.id,
                        'journal_id': 1,
                        'ref': invoice_line.invoice_id.number,
                        'move_id':move_line_id,
                        'invoice_line_id':invoice_line.id,
                    }
                    vals_lines.update({'date':start_date})
                    #print 'vals', vals_lines
                    analytic_line_obj.create(cr, uid, vals_lines)
                    count +=1
                    for i in range(1, month_qty):
                        count +=1
                        start_date += DateTime.RelativeDateTime(months=1)
                        #print start_date
                        vals_lines.update({'date':start_date})
                        analytic_line_obj.create(cr, uid, vals_lines) #TODO check month_qty

        #print 'count', count

        return count

account_move_line()
