# -*- encoding: utf-8 -*-
##############################################################################
#
#    Fleet maintenance module for OpenERP
#    Copyright (C) 2010-2012 Alexis de Lattre <alexis@via.ecp.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

# A SAVOIR : is_maintenance est à False au début dans product_id_change !!!
# TODO :
# Tester la création d'une credit note
# Si on change une date d'anniversaire, ça marche pas du tout, car on lit quand même l'ancienne date -> cf fonction _anniversary_time dans stock.py
# Solutionner l'incompatibilité entre le code de action_invoice_create et les contraintes sur la qté
# Actuellement, on peut changer changer le partenaire sur une sale order avec maintenance et sauver... il ne proteste pas alors que la flotte appatient à un autre !!!
# PB qd la maintenance_product_qty est à 0 -> la qté n'est pas mise à jour

# NICE
# date de début proposée par défaut lors de la création d'une invoice moins éloignée 
# dans le temps
# Trouver une meilleure solution pour éviter de recalculer le nb de mois dans le code des contraintes

from osv import osv, fields
from tools.translate import _
from datetime import datetime
from dateutil.relativedelta import relativedelta

fixed_month_init_day = 1 # TODO make this a fields.property parameter!
fixed_days_before_month_end = 0 #TODO make this a fields.property parameter!
min_maintenance_months = 6 #TODO make this a fields.property parameter!
default_maintenance_start_delta = 3 #TODO make this a fields.property parameter!


class fleet_maintenance_osv(osv.osv):
    _register = False

    def common_product_id_change(self, cr, uid, ids, result, params, context=None):
        if context is None:
            context = {}
        if not result.get('value', False):
            result['value'] = {}
        if params['product_id']:
            product_obj = self.pool.get('product.product').browse(cr, uid, params['product_id'])
            #print "product_obj.is_maintenance = ", product_obj.is_maintenance
            if product_obj.is_maintenance:
                is_maintenance_dict = {'is_maintenance': True}
                if not params['maintenance_product_qty']:
                    is_maintenance_dict['maintenance_product_qty'] = 1
                result['value'].update(is_maintenance_dict)
                params.update(is_maintenance_dict)
                if self == self.pool.get('account.invoice.line'):
                    result['value'].update({'account_analytic_id': product_obj.maintenance_analytic_id.id})
            else:
                params.update({'is_maintenance': False, 'maintenance_product_qty': False})
                result['value'].update({
                    'is_maintenance': False,
                    'maintenance_start_date': False,
                    'maintenance_end_date': False,
                    'maintenance_product_qty': False,
                    })
            if self._compute_subfleet_required(cr, uid, params['product_id'], params.get('invoice_type', False), context=context):
                result['value'].update({'subfleet_required': True})
                params.update({'subfleet_required': True})
            else:
                result['value'].update({'subfleet_required': False, 'fleet_id': False})
                params.update({'subfleet_required': False})
            # If the previous product was already a maintenance product ???? | only for product_machin_qty
            if params['is_maintenance'] and params['maintenance_product_qty'] and params['maintenance_month_qty']:
                #print "ONCHANGE product_id_change do we enter in the IF ?"
                result['value'].update({'quantity': params['maintenance_product_qty'] * params['maintenance_month_qty']}) # TODO : changer quantity by product_uom_qty when sale
               #result['value'].update({'product_uos_qty': maintenance_product_qty * maintenance_month_qty}) # TODO * product_obj.uos_coeff
                #result['warning'] = {'title': 'Maintenance Quantity Warning', 'message': "For maintenance products, you should use instead the maintenance quantity from the other tab to compute this field"} # The field will be invisible, so no need for a warning
        # We need this when we switch from to regular product to a maintenance product
        #print "ONCHANGE PRODUCT_ID_CHANGE result_avant_fleet_id_change = ", result
        # Il faut REFAIRE LE PARAM
        #
        fleet_result = self.common_fleet_id_change(cr, uid, ids, params, context=context)
        result['value'].update(fleet_result['value'])
        #print "ONCHANGE PRODUCT_ID_CHANGE result_apres_fleet_id_change = ", result
        return result

    def common_maintenance_qty_date_change(self, cr, uid, ids, params, context=None):
        result = {}
        if not params['is_maintenance']:
            return result

        result['value'] = {}
        warning_messages = ""

        if params['maintenance_start_date']:
            start = datetime.strptime(params['maintenance_start_date'], '%Y-%m-%d')
            if start.day != fixed_month_init_day:
                warning_messages += "- Start date should ideally start at day %s of the month; corrected to day %s\n" % (fixed_month_init_day, fixed_month_init_day)
                start = datetime(start.year, start.month, fixed_month_init_day)
            result['value'].update({'maintenance_start_date': start.strftime('%Y-%m-%d')})

        if params['maintenance_end_date']:
            end = datetime.strptime(params['maintenance_end_date'], '%Y-%m-%d')
            en_date_check = end + relativedelta(days=(fixed_days_before_month_end + 1))
            if end.month == en_date_check.month or en_date_check.day != 1:
                warning_messages += "- End date should be %s days before the last day of the month ! It has been reset to the correct value.\n" % fixed_days_before_month_end
                end = end + relativedelta(day=(31-fixed_days_before_month_end))
                result['value'].update({'maintenance_end_date': end.strftime('%Y-%m-%d')})

        if params['maintenance_start_date'] and params['maintenance_end_date']:
            if end < start:
                warning_messages += "- End date should be AFTER start date !\n"
                end = start + relativedelta(day=(31-fixed_days_before_month_end))
                result['value'].update({'maintenance_end_date': end.strftime('%Y-%m-%d')})

            params['maintenance_month_qty'] = self._get_maintenance_month_qty_from_start_end(cr, uid, start, end, context=context)

            result['value'].update({'maintenance_month_qty': params['maintenance_month_qty']})
            if params['maintenance_month_qty'] < min_maintenance_months:
                warning_messages += "- we usually try to sell %s months at least!\n" % min_maintenance_months

            if params['fleet_id']:
                fleet = self.pool.get('stock.location').browse(cr, uid, params['fleet_id'])
                theoretic_end = self._get_end_date_from_start_date(cr, uid, start, fleet, context=context)
                if theoretic_end.month != end.month:
                    warning_messages += "- The end month (%s) doesn't correspond to the anniversary month of the fleet (%s) !\n" % (end.strftime('%B'), theoretic_end.strftime('%B'))
                elif theoretic_end.year != end.year or theoretic_end.day != end.day:
                    warning_messages += "- Theoretic maintenance end date was: %s !\n" % theoretic_end.strftime('%Y-%m-%d')

        if params['maintenance_product_qty'] and params['maintenance_month_qty']:
            result['value'].update({'quantity': params['maintenance_product_qty'] * params['maintenance_month_qty']}) # WARNING : override this value for sale.order
            #result['value'].update({'product_uos_qty': params['maintenance_product_qty'] * params['maintenance_month_qty']}) # TODO * product_obj.uos_coeff

        if len(warning_messages) > 1:
            result['warning'] = {'title': 'Maintenance dates warning', 'message': warning_messages}
        #print "ONCHANGE MAINT_QTY_DATE result = ", result
        return result

    def common_fleet_id_change(self, cr, uid, ids, params, context=None):
        '''Apply the rules for default start and end date and default fleet'''
        result = {}
        result['value'] = {}
        #print "ONCHANGE params fleet_id START =", params.get('fleet_id', False)
        #print "ONCHANGE params default_fleet_id START =", params.get('default_fleet_id', False)
        fleet_id = params.get('fleet_id', False)
        #print "ONCHANGE fleet_id params =", fleet_id
        #print "ONCHANGE subfleet_required params =", params.get('subfleet_required', False)
        #print "ONCHANGE default_fleet_id params =", params.get('default_fleet_id', False)
        if not fleet_id and params.get('subfleet_required', False):
            fleet_id = params.get('default_fleet_id', False)
        #print "ONCHANGE fleet_id RESULT =", fleet_id

        if fleet_id:
            fleet = self.pool.get('stock.location').browse(cr, uid, fleet_id)
            # Fleet has an anniversary month and is currently under maintenance
            # -> we set the start date as the next anniversary date
            if fleet.expire_time and fleet.is_under_maintenance:
                start_date = datetime.strptime(fleet.expire_time, '%Y-%m-%d') + relativedelta(days=(fixed_days_before_month_end + 1))
            else:
                # For all other fleets
                # -> we set the start date as the default maintenance start date
                start_date = params.get('maintenance_start_date', False) and datetime.strptime(params.get('maintenance_start_date', False), '%Y-%m-%d') or datetime.strptime(self.default_maintenance_start_date(cr, uid, context=context), '%Y-%m-%d')
            # We set end date as default end date
            end_date = self._get_end_date_from_start_date(cr, uid, start_date, fleet, context=context)

            result['value'].update({'fleet_id': fleet_id, 'parent_fleet_id': fleet.location_id.id})

            if params['product_id'] and params['is_maintenance']:
                    maintenance_month_qty = self._get_maintenance_month_qty_from_start_end(cr, uid, start_date, end_date, context=context)
                    result['value'].update({
                        'maintenance_start_date': start_date.strftime('%Y-%m-%d'),
                        'maintenance_end_date': end_date.strftime('%Y-%m-%d'),
                        'maintenance_month_qty': maintenance_month_qty,
                        'quantity': params['maintenance_product_qty'] * maintenance_month_qty
                        }) # WARNING pr sale.order
                    #result['value'].update({'product_uos_qty': maintenance_product_qty * maintenance_month_qty})
        #print "ONCHANGE FLEET_ID result = ", result
        return result

    def _get_end_date_from_start_date(self, cr, uid, start_date, sub_fleet, context=None):
        '''Compute the default maintenance end date
        :param browse_record sub_fleet: the sublfeet
        :param datetime start_date: the start date
        :return: datetime
        '''
        #print "END_FROM_START sub_fleet =", sub_fleet.id
        anniversary_time = datetime.strptime(sub_fleet.anniversary_time, '%Y-%m-%d')
        #print "END_FROM_START anniv =", anniversary_time
        anniv_month = anniversary_time.month
        next_anniv = datetime(start_date.year, anniversary_time.month, 1) + relativedelta(day=(31-fixed_days_before_month_end))
        maintenance_month_qty = self._get_maintenance_month_qty_from_start_end(cr, uid, start_date, next_anniv, context=context)
        # We don't want to sell less than "min_maintenance_months" of maintenance
        # So, in case we have less that "min_maintenance_months", we want to sell maintenance
        # until next anniversary + 1 year
        if maintenance_month_qty < min_maintenance_months:
            end = datetime(start_date.year + 1, anniv_month, 1) + relativedelta(day=(31-fixed_days_before_month_end))
        else:
            end = next_anniv
        return end

    def _get_maintenance_month_qty_from_start_end(self, cr, uid, start, end, context=None):
        '''Compute the number of months between maintenance start and end dates
        :param datetime start: start date
        :param datetime end: end date
        :return integer: number of months
        '''
        delta = relativedelta(end + relativedelta(days=(fixed_days_before_month_end + 1)), start)
        return delta.months + delta.years * 12

    def default_maintenance_start_date(self, cr, uid, context=None):
        now = datetime.now()
        start_date = datetime(now.year, now.month, fixed_month_init_day) + relativedelta(months=default_maintenance_start_delta)
        return start_date.strftime('%Y-%m-%d')

    def function_maintenance_month_qty(self, cr, uid, ids, context=None):
        '''Used for definition of maintenance_month_qty field, which is a fields.function'''
        result = {}
        for line in self.browse(cr, uid, ids, context=context):
            if line.maintenance_start_date and line.maintenance_end_date and line.product_id.is_maintenance:
                result[line.id] = self._get_maintenance_month_qty_from_start_end(cr, uid, datetime.strptime(line.maintenance_start_date, '%Y-%m-%d'), datetime.strptime(line.maintenance_end_date, '%Y-%m-%d'), context=context)
            else:
                result[line.id] = False
        return result

    def _get_end_date_from_start_date_and_months(self, cr, uid, start_date, maintenance_month_qty, context=None):
        '''
        :param datetime start_date: start date
        :param integer maintenance_month_qty
        :return datetime end_date
        '''
        return start_date + relativedelta(months=maintenance_month_qty, days=-1)

    def function_subfleet_required(self, cr, uid, ids, context=None):
        '''Used for definition of subfleet_required, which is a fields.function'''
        result = {}
        invoice_type = False
        for line in self.browse(cr, uid, ids, context=context):
            if self == self.pool.get('account.invoice.line'):
                invoice_type = line.invoice_id.type
            result[line.id] = self._compute_subfleet_required(cr, uid, line.product_id.id, invoice_type, context=context)
        #print "ONCHANGE function_subfleet_required result =", result
        return result

    def _compute_subfleet_required(self, cr, uid, product_id, invoice_type=False, context=None):
        '''Used by many other functions, to avoid code duplication'''
        #print "ONCHANGE _compute_subfleet_required product_id =", product_id
        #print "ONCHANGE _compute_subfleet_required inv_type=", invoice_type
        if product_id:
            if invoice_type in ('in_invoice', 'in_refund'):
                return False
            product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
            if product.is_maintenance or product.type == 'product' or product.type == 'consu':
                return True
            else:
                return False
        else:
            return False

    def contraints_check_maintenance_all(self, cr, uid, ids, order_confirm=False, context=None):
        #print "contraints_check_maintenance_all ids=%s, order_confirm=%s, object=%s" %(unicode(ids), str(order_confirm), self._name)
        msg_title = "Error on maintenance data"
        for line in self.browse(cr, uid, ids, context=context):
            subfleet_required = self._compute_subfleet_required(cr, uid, line.product_id.id, context=context)
            if self == self.pool.get('account.invoice.line'):
                # Check if account_invoice_layout is installed
                if self.pool.get('ir.model.fields').search(cr, uid, [('name', '=', 'state'), ('model', '=', 'account.invoice.line')])!=[]:
                    # If it's not a "regular" line
                    if line.state != 'article':
                        continue
                qty = line.quantity
                line_parent_partner_id = line.invoice_id.partner_id
                # the constraints below should only be applied to customer invoices/refunds
                if line.invoice_id.type == ('in_invoice' or 'in_refund'):
                    continue
                # in purchase/purchase.py function action_invoice_create
                # they first create the invoice line and AFTER they create
                # the invoice... so, when the invoice line is created,
                # line.invoice_id is False... so it doesn't match above
                # and it blocks the creation of the invoice from the PO
                # As a workaround, I check the account type !
                # You can say "it's dirty !", but do you have a better idea ?
                if line.account_id.user_type.report_type == 'expense':
                    continue
            if self == self.pool.get('sale.order.line'):
                # If fleet_required has been disabled on sale shop
                # we disable the check to see if fleet_id is not False
                if not line.order_id.shop_id.fleet_required and not order_confirm:
                    subfleet_required = False
                # Check if sale_layout is installed
                if self.pool.get('ir.model.fields').search(cr, uid, [('name', '=', 'layout_type'), ('model', '=', 'sale.order.line')])!=[]:
                    if line.layout_type != 'article':
                        continue
                qty = line.product_uom_qty
                line_parent_partner_id = line.order_id.partner_id
                # If it's a loan line, we don't go any further
                if self.pool.get('ir.model.fields').search(cr, uid, [('name', '=', 'is_loan'), ('model', '=', 'sale.order')]) != []: # the is_loan field is added by product_loan module
                    if line.order_id.is_loan:
                        continue
                # I can't test this on invoice lines because, when the invoice is created from a sale order, the
                # invoice lines are created BEFORE the invoice, so line_parent_partner_id is False during a small
                # amount of time
                if subfleet_required:
                    if line.fleet_id and line.fleet_id.location_id.partner_id != line_parent_partner_id:
                        raise osv.except_osv(msg_title, "The Sub Fleet '%s' belongs to the partner '%s' but it should belong to the selected partner '%s'" %(line.fleet_id.name, line.fleet_id.location_id.partner_id.name, line_parent_partner_id.name))

            if subfleet_required:
                if not line.fleet_id:
                    raise osv.except_osv(msg_title, "The Sub Fleet is missing on the line with product '%s'" %line.product_id.name)
                if line.fleet_id.fleet_type != 'sub_fleet':
                    raise osv.except_osv(msg_title, "The Sub Fleet '%s' is not of type 'subfleet'" % line.fleet_id.name)

            if line.product_id.is_maintenance:
                # At this stage of the code, the value of "maintenance_month_qty" hasn't been
                # computed yet, so we recompute it here :-(
                months = (self.function_maintenance_month_qty(cr, uid, [line.id], context=context)).get(line.id, False)
                #print "ONCHANGE CONSTRAINT months = ", months
                #print "ONCHANGE CONSTRAINT qty = ", qty
                #print "ONCHANGE CONSTRAINT line.maintenance_product_qty =", line.maintenance_product_qty
                if qty != line.maintenance_product_qty * months:
                    #print "ONCHANGE CONSTRAINT line.maintenance_month_qty =", line.maintenance_month_qty
                    #print "ONCHANGE CONSTRAINT line.maintenance_start_date =", line.maintenance_start_date
                    #print "ONCHANGE CONSTRAINT line.maintenance_end_date =", line.maintenance_end_date
                    # TODO : ça pose pb lors du create invoice (seul le champ qty a une valeur ds un premier temps)
                    raise osv.except_osv(msg_title, "For a maintenance product, we should have : quantity = maintenance month quantity * maintenance product quantity. It is not the case on the line with '%s'" % line.product_id.name)
                if not line.maintenance_start_date:
                    raise osv.except_osv(msg_title, "The 'maintenance start date' is missing on the line with '%s'" % line.product_id.name)
                if not line.maintenance_end_date:
                    raise osv.except_osv(msg_title, "The 'maintenance end date' is missing on the line with '%s'" % line.product_id.name)
            else:
                if line.maintenance_start_date:
                    raise osv.except_osv(msg_title, "The 'maintenance start date' should not have a value on the line with '%s'" % line.product_id.name)
                if line.maintenance_end_date:
                    raise osv.except_osv(msg_title, "The 'maintenance end date' should not have a value on the line with '%s'" % line.product_id.name)
                if line.maintenance_product_qty:
                    raise osv.except_osv(msg_title, "The 'maintenance product quantity' should be '0' on the line with '%s'" % line.product_id.name)

        return True


    def _common_update_with_defaultfleet(self, cr, uid, parent_self, parent_ids, context=None):
        cur_obj = parent_self.browse(cr, uid, parent_ids[0], context=context)
        new_default_fleet_id = cur_obj.default_fleet_id.id
        if not new_default_fleet_id:
            raise osv.except_osv(_('Error :'), _("Please set a Default Sub-Fleet."))
        line_ids_to_update = []
        if parent_self == self.pool.get('sale.order'):
            loop_on = cur_obj.order_line
        elif parent_self == self.pool.get('account.invoice'):
            loop_on = cur_obj.invoice_line
        else:
            raise
        for line in loop_on:
            if self._compute_subfleet_required(cr, uid, line.product_id.id, context=context):
                line_ids_to_update.append(line.id)
        #print "line_ids_to_update=", line_ids_to_update
        if line_ids_to_update:
            self.write(cr, uid, line_ids_to_update,
                {'fleet_id': new_default_fleet_id}, context=context)
        return True
