# -*- encoding: utf-8 -*-
##############################################################################
#
#    Fleet maintenance module for OpenERP
#    Copyright (C) 2008-2010 Anevia S.A. (www.anevia.com),
#    @author Raphaël Valyi <rvalyi@akretion.com>
#    Copyright (C) 2010 Alexis de Lattre <alexis@via.ecp.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from onchange import fleet_maintenance_osv

fixed_month_init_day = 1 # TODO make this a fields.property parameter!
fixed_days_before_month_end = 0 #TODO make this a fields.property parameter!
min_maintenance_months = 6 #TODO make this a fields.property parameter!
default_maintenance_start_delta = 3 #TODO make this a fields.property parameter!


class sale_shop(osv.osv):
    _inherit = "sale.shop"

    _columns = {
        'fleet_required': fields.boolean('Fleets are required even on quotations', help="This option is checked by default. If you import sale orders from an external system that doesn't have the fleet information, you should disable this option, so that the constraint on sale.order.line doesn't block the import."),
        }

    _defaults = {
        'fleet_required': True,
        }

sale_shop()


class sale_order_line(fleet_maintenance_osv):
    _inherit = "sale.order.line"

    def create(self, cr, uid, vals, context=None):
        """Prevents the ORM from trying to write the is_maintenance product fields view fields related.
        User might indeed not have this right, example is a salesman duplicating a sale_order.
        Actually this looks like a bug in OpenObject because it will try to write even if we declared
        the is_maintenance field as read_only.
        see bugreport here https://bugs.launchpad.net/openobject-server/+bug/426676
        """
        if vals.get('is_maintenance', False):
            del(vals['is_maintenance'])
        return super(sale_order_line, self).create(cr, uid, vals, context=context)


    def _function_maintenance_month_qty(self, cr, uid, ids, prop, unknow_none, context=None):
        '''Used for definition of maintenance_month_qty field, which is a fields.function'''
        return self.function_maintenance_month_qty(cr, uid, ids, context=context)

    def _function_subfleet_required(self, cr, uid, ids, prop, unknow_none, context=None):
        '''Used for definition of subfleet_required, which is a fields.function'''
        return self.function_subfleet_required(cr, uid, ids, context=context)

    def _get_sale_order_line_from_stock_location(self, cr, uid, ids, context=None):
        return self.pool.get('sale.order.line').search(cr, uid, [('fleet_id', 'in', ids)], context=context)

    _columns = {
        'maintenance_product_qty': fields.integer('Maintenance Product Quantity', help="Number of 'real' products which will be covered by this maintenance product."),
        'maintenance_month_qty': fields.function(_function_maintenance_month_qty, string="Maintenance Month Quantity", type='integer', store={
            'sale.order.line': (lambda self, cr, uid, ids, c={}: ids, ['maintenance_start_date', 'maintenance_end_date'], 10),
            }, help="Number of months of maintenance."),
        'fleet_id': fields.many2one('stock.location', 'Sub Fleet', help="For a 'real' product : this is the sub-fleet where the product will be sent to. For a 'maintenance product' : this is the sub-fleet which will be is covered by the maintenance."),
        'parent_fleet_id': fields.related('fleet_id', 'location_id', type='many2one', relation='stock.location', string='Fleet', store={
            'sale.order.line': (lambda self, cr, uid, ids, c={}: ids, ['fleet_id'], 10),
            'stock.location': (_get_sale_order_line_from_stock_location, ['location_id'], 20),
            }, help="Fleet to which the Sub-fleet belongs"),
        # We need store=True on parent_fleet_id, because othewise the field
        # fleet_sale_order_lines on stock.location doesn't work
        'maintenance_start_date': fields.date('Maintenance Start Date', help="Date when the maintenance starts."),
        'maintenance_end_date': fields.date('Maintenance End Date', help="Date when the maintenance stops."),
        'is_maintenance': fields.related('product_id', 'is_maintenance', type='boolean', string='Is Maintenance', readonly=True), # Not displayed on screen
        'subfleet_required': fields.function(_function_subfleet_required, string='Sub-Fleet required', type='boolean', store={
            'sale.order.line': (lambda self, cr, uid, ids, c={}: ids, ['product_id'], 10),
            }), # Not displayed on screen
    }


    def _convert_quantity(self, cr, uid, result, context=None):
        '''in "result", the product quantity is stored under "quantity", so we have to copy it
        to the right field name'''
        if result:
            if result.get('value', False):
                if result['value'].get('quantity', False):
                    result['value'].update({'product_uom_qty': result['value'].get('quantity', 0)})
                    result['value'].update({'product_uos_qty': result['value'].get('quantity', 0)}) # TODO * product_obj.uos_coeff
                    del((result['value'])['quantity'])
        return result


    def maintenance_qty_date_change(self, cr, uid, ids, maintenance_product_qty=False, maintenance_month_qty=False, maintenance_start_date=False, maintenance_end_date=False, is_maintenance=False, fleet_id=False, context=None):
        params = {'maintenance_product_qty': maintenance_product_qty, 'maintenance_month_qty': maintenance_month_qty, 'maintenance_start_date': maintenance_start_date, 'maintenance_end_date': maintenance_end_date, 'is_maintenance': is_maintenance, 'fleet_id': fleet_id}
        #print "SALE params maintenance_date_qty = ", params
        result = self.common_maintenance_qty_date_change(cr, uid, ids, params, context=context)
        result_converted_qty = self._convert_quantity(cr, uid, result, context=context)
        #print "SALE MAINT_QTY_DATE_CHANGE result_converted_qty = ", result_converted_qty
        return result_converted_qty


    def fleet_id_change(self, cr, uid, ids, is_maintenance=False, default_fleet_id=False, fleet_id=False, product_id=False, maintenance_start_date=False, maintenance_product_qty=False, subfleet_required=False, context=None):
        params = {'is_maintenance': is_maintenance, 'default_fleet_id': default_fleet_id, 'fleet_id': fleet_id, 'product_id': product_id, 'maintenance_start_date': maintenance_start_date, 'maintenance_product_qty': maintenance_product_qty, 'subfleet_required': subfleet_required}
        #print "SALE params fleet_id_change =", params
        result = self.common_fleet_id_change(cr, uid, ids, params, context=context)
        result_converted_qty = self._convert_quantity(cr, uid, result, context=context)
        #print "SALE FLEET_ID_CHANGE result_converted_qty = ", result_converted_qty
        return result_converted_qty


    #TODO adapt signature to new fiscal_position parameter
    def product_id_change(self, cr, uid, ids, pricelist, product_id, qty=0,
            uom=False, qty_uos=0, uos=False, name='', partner_id=False,
            lang=False, update_tax=True, date_order=False, packaging=False,
            fiscal_position=False, flag=False, is_maintenance=False,
            maintenance_product_qty=False, maintenance_month_qty=False,
            default_fleet_id=False, subfleet_required=False, context=None):

        result = super(sale_order_line, self).product_id_change(cr, uid, ids, pricelist,
            product_id, qty, uom, qty_uos, uos, name, partner_id, lang, update_tax, date_order, packaging, fiscal_position, flag, context=context)
        #print "SALE PRODUCT_ID_CHANGE result SUPER = ", result
        #print "SALE product_id_change CONTEXT = ", context
        params = {'product_id': product_id, 'is_maintenance': is_maintenance, 'maintenance_product_qty': maintenance_product_qty, 'maintenance_month_qty': maintenance_month_qty, 'default_fleet_id': default_fleet_id, 'subfleet_required': subfleet_required}
        #print "SALE product_id_change PARAMS =", params
        result = self.common_product_id_change(cr, uid, ids, result, params, context=context)
        result_converted_qty = self._convert_quantity(cr, uid, result, context=context)
        #print "SALE PRODUCT_ID_CHANGE result = ", result_converted_qty
        return result_converted_qty


    def _prepare_order_line_invoice_line(self, cr, uid, line, account_id=False, context=None):
        """Inherit to add maintenance-specific fields
            This function is called when you invoice based on deliveries for service lines
            and it is called when you invoice from the sale order for all lines
        """
        il_vals = super(sale_order_line, self)._prepare_order_line_invoice_line(cr, uid, line,
            account_id=account_id, context=context)
        if line.product_id and line.product_id.is_maintenance:
            il_vals.update({
                'maintenance_start_date': line.maintenance_start_date,
                'maintenance_end_date': line.maintenance_end_date,
                'maintenance_product_qty': line.maintenance_product_qty,
                # WARNING : for this to work, you need to patch
                # addons-6.1/sale/stock.py line 182 and comment-out the line
                # vals['account_analytic_id'] = self._get_account_analytic_invoice(cursor, user, picking, sale_line)
                'account_analytic_id':
                    line.product_id.maintenance_analytic_id.id,
                })
        if line.product_id and line.fleet_id:
            il_vals.update({
                'fleet_id': line.fleet_id.id,
            })
        return il_vals


    def _check_maintenance_all(self, cr, uid, ids, context=None):
        return self.contraints_check_maintenance_all(cr, uid, ids, context=context)


    _constraints = [
        (_check_maintenance_all, "We raise the error msgs inside the code",
            ['product_uom_qty', 'maintenance_product_qty', 'maintenance_month_qty', 'maintenance_start_date', 'maintenance_end_date', 'fleet_id', 'product_id']),
            ]

sale_order_line()



class sale_order(osv.osv):
    _inherit = "sale.order"

    _columns = {
        'default_fleet_id': fields.many2one('stock.location', 'Default Sub-Fleet'),
    }


    def _prepare_order_line_move(self, cr, uid, order, line, picking_id, date_planned, context=None):
        move_line_vals = super(sale_order, self)._prepare_order_line_move(cr, uid, order, line, picking_id, date_planned, context=context)
        if line.fleet_id:
            move_line_vals.update({'location_dest_id': line.fleet_id.id})
        return move_line_vals


    def _prepare_invoice(self, cr, uid, order, lines, context=None):
        """ Copy Default Sub-fleet from sale order to invoice """
        invoice_vals = super(sale_order, self)._prepare_invoice(cr, uid, order, lines, context=context)
        if order.default_fleet_id:
            invoice_vals.update({'default_fleet_id': order.default_fleet_id.id})
        return invoice_vals


    def update_with_defaultfleet(self, cr, uid, ids, context=None):
        return self.pool.get('sale.order.line')._common_update_with_defaultfleet(cr, uid, self, ids, context=context)


    def action_wait(self, cr, uid, ids, context=None):
        """When order.shop_id.fleet_required=False, the constraint on
        sale.order.line is not fully performed, so we perform the full
        constraint when we confirm the order"""
        for order in self.browse(cr, uid, ids, context=context):
            if order.shop_id.fleet_required:
                continue
            else:
                for line in order.order_line:
                    self.pool.get('sale.order.line').contraints_check_maintenance_all(cr, uid, [line.id], order_confirm=True, context=context)

        return super(sale_order, self).action_wait(cr, uid, ids, context=context)

sale_order()
