# -*- coding: utf-8 -*-
##############################################################################
#
#    Fleet maintenance module for OpenERP
#    Copyright (C) 2012 Akretion (http://www.akretion.com/)
#    @author Alexis de Lattre <alexis.delattre@akretion.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

# TODO what should we do if maintenance lines already exist ?
# passer la flotte ds le maintenance_date_change

from osv import fields, osv
from tools.translate import _
from datetime import datetime

class sale_add_maintenance_lines(osv.osv_memory):
    _name = "sale.add.maintenance.lines"
    _description = "Wizard to add maintenance lines"

    _columns = {
        'maintenance_start_date': fields.date('Maintenance Start Date', required=True, help="This will be the 'Maintenance start date' for the maintenance lines that will be generated."),
        'maintenance_end_date': fields.date('Maintenance End Date', required=True, help="This will be the 'Maintenance end date' for the maintenance lines that will be generated."),
        'discount': fields.float('Discount (%)', digits=(16, 2)),
        'is_discount_from_pricelist': fields.boolean('Use Pricelist for Discount'),
        }


    def _get_default_end_date(self, cr, uid, context=None):
        so_obj = self.pool.get('sale.order')
        sol_obj = self.pool.get('sale.order.line')
        if not context.get('active_id'):
            return False
        order = so_obj.browse(cr, uid, context.get('active_id'), context=context)
        start_date = datetime.strptime(sol_obj.default_maintenance_start_date(cr, uid, context=context), '%Y-%m-%d')
        if order.default_fleet_id:
            end_date = sol_obj._get_end_date_from_start_date(cr, uid, start_date, order.default_fleet_id, context=context)
            return end_date.strftime('%Y-%m-%d')
        else:
            return False

    _defaults = {
        'maintenance_start_date': lambda self, cr, uid, c: self.pool.get('sale.order.line').default_maintenance_start_date(cr, uid, context=c),
        'maintenance_end_date': _get_default_end_date,
        'is_discount_from_pricelist': True,
        }

    def maintenance_date_change(self, cr, uid, ids, maintenance_start_date, maintenance_end_date, context=None):
        sol_obj = self.pool.get('sale.order.line')
        params = {'maintenance_product_qty': False, 'maintenance_month_qty': False, 'maintenance_start_date': maintenance_start_date, 'maintenance_end_date': maintenance_end_date, 'is_maintenance': True, 'fleet_id': False}
        result = sol_obj.common_maintenance_qty_date_change(cr, uid, ids, params, context=context)
        return result


    def add_maintenance_lines(self, cr, uid, ids, context=None):
        '''Wizard to add maintenance lines'''
        so_obj = self.pool.get('sale.order')
        sol_obj = self.pool.get('sale.order.line')
        mywiz = self.browse(cr, uid, ids[0], context=context)
        start_date_str = mywiz.maintenance_start_date
        end_date_str = mywiz.maintenance_end_date
        start_date = datetime.strptime(start_date_str, '%Y-%m-%d')
        end_date = datetime.strptime(end_date_str, '%Y-%m-%d')
        maintenance_month_qty = sol_obj._get_maintenance_month_qty_from_start_end(cr, uid, start_date, end_date, context=context)
        order = so_obj.browse(cr, uid, context['active_id'], context=context)
        maintenance_lines_to_create = []
        for oline in order.order_line:
            if oline.product_id.type in ('product', 'consu') and oline.product_id.maintenance_template:
                product_ids = self.pool.get('product.product').search(cr, uid, [('product_tmpl_id', '=', oline.product_id.maintenance_template.id)], context=context)
                if not product_ids: raise # This should never happen
                maintenance_lines_to_create.append({
                    'product_id': product_ids[0], # TODO when we will manage several levels of maintenance, we will have to change this code
                    'maintenance_product_qty': oline.product_uom_qty,
                    'fleet_id': oline.fleet_id.id,
                    'maintenance_start_date': start_date_str,
                    'maintenance_end_date': end_date_str,
                    'maintenance_month_qty': maintenance_month_qty,
                    'order_id': order.id,
                    })
        for maintenance_line_vals in maintenance_lines_to_create:
            product_id_change_vals = sol_obj.product_id_change(cr, uid, context['active_id'], order.pricelist_id.id, maintenance_line_vals.get('product_id'), partner_id=order.partner_id.id, date_order=order.date_order, fiscal_position=order.fiscal_position.id, is_maintenance=True, maintenance_product_qty=maintenance_line_vals.get('maintenance_product_qty'), maintenance_month_qty=maintenance_month_qty, context=context)['value']
            tax_id_val = []
            if product_id_change_vals.get('tax_id'):
                for tax_id in product_id_change_vals.get('tax_id'):
                    tax_id_val.append((4, tax_id))
            maintenance_line_vals.update({
                'name': product_id_change_vals.get('name'),
                'price_unit': product_id_change_vals.get('price_unit'),
                'notes': product_id_change_vals.get('notes'),
                'tax_id': tax_id_val,
                'product_uom': product_id_change_vals.get('product_uom'),
                'product_uom_qty': product_id_change_vals.get('product_uom_qty'),
                'product_uos': product_id_change_vals.get('product_uos'),
                    })
            # Handle discount
            if mywiz.is_discount_from_pricelist:
                maintenance_line_vals['discount'] = product_id_change_vals.get('discount')
            else:
                maintenance_line_vals['discount'] = mywiz.discount
            self.pool.get('sale.order.line').create(cr, uid, maintenance_line_vals,
                context=context)


        return {'type': 'ir.actions.act_window_close'}
